# CipherMail Community Gateway

This repository contains the 'open core' code that powers the CipherMail Email
Encryption Gateway. It contains everything you need to build the libraries and
OS packages of the Community Edition.

For more information on the Enterprise Edition and paid support, visit our
website <https://www.ciphermail.com/>.

## End-user documentation

This README is meant to offer help with developing, testing and releasing
CipherMail Email Encryption Gateway. For installation instructions, operational
guides and other end-user documentation, visit
<https://www.ciphermail.com/documentation>

## Setting things up

1.  Install dependencies. On Debian GNU/Linux, this should install everything
    you need:
    ```
    # apt install \
        ant ant-optional build-essential cdbs debhelper docker.io \
        docker-compose fakeroot git gnupg rpm
    ```
2.  Start the Docker service containers: `$ docker-compose up -d`. Note that
    your Unix user account needs to be in the `docker` group for this to work.

## Unit testing

The back-end unit tests can be started using Ant: `$ ant test-gw-backend`.
There are currently no front-end unit tests available.

Some testing scenarios require setting certain system properties. These system
properties must be configured through arguments of the JUnit JVM, which is
possible through the `ciphermail.junit.opts` Ant property.

For example, if you want to test against MariaDB instead of the default
PostgreSQL database, set the corresponding JUnit JVM system property like
this:

```
$ ant \
    -Dciphermail.junit.opts="-Dciphermail.hibernate.database.type=mysql" \
    test-gw-backend
```

Furthermore, you can pick which unit tests are performed by overriding the
JUnit 'batchtest' include pattern like this:

```
$ ant \
    -Dciphermail.junit.batchtest-include="mitm/application/djigzo/dlp/AllTests.java" \
    test-gw-backend
```

Note that running all tests inside a Java test package is also possible by
using the `**/*` pattern, but this will cause most tests to be run twice, which
will cause issues with JUnit due to test objects being loaded multiple times.

### Testing proxy client support with Squid

The Docker Compose setup includes a Squid proxy server, which supports both
HTTP and HTTPS proxying and requires HTTP basic authentication. It is used to
test the CipherMail proxy client support. You can use curl to check if the
Squid server works correctly:
```
$ curl --proxy http://127.0.0.1:3128 --proxy-user test:test \
  https://www.ciphermail.com/
```

## Running the backend

Locally running the backend is possible with `$ ant start-gw-backend`

## Running the front-end

Runnning the front-end locally requires a servlet engine like Tomcat or Jetty. To run the front-end locally for 
development purposes, using Jetty is recommended.

Downloaded a Jetty 9 or 10 tgz releases from <http://www.eclipse.org/jetty/download.php>.

**NOTE:**  At the moment, the provided jetty config will only work with Jetty 9 and 10.

Unpack the tgz file to some location.

To start Jetty locally

`$ ant start-gw-frontend -Djetty.home=/temp/jetty-distribution-9.4.36.v20210114`

`jetty.home` should point to the directory where jetty was unpacked (the path should be a fully qualified).

After Jetty is started, you can access the admin GUI at <http://127.0.0.1:8080> and the portal GUI at 
<http://127.0.0.1:8080/web>.

To login, use your Unix system account (your Unix system account should have sudo permissions).

## Building packages

You can build all Debian and RPM packages using `$ ant release`. The packages
will end up in `./build/packages`.

## License

- This program contains software created by the Exolab Project
  (http://www.exolab.org/).

- This product includes software developed by the Visigoth Software Society
  (http://www.visigoths.org/).

- This program contains software created by the DOM4J Project
  (http://www.dom4j.org).

- This product includes software developed by the MX4J project
  (http://mx4j.sourceforge.net).

- This product includes software developed by the RealityForge Group
  (http://www.realityforge.org/).

- This product includes icons by FamFamFam
  (http://www.famfamfam.com/lab/icons/silk/).

- This product includes icons by Glyphicons
  (https://www.glyphicons.com/).

Copyright (C) 2008-2022  CipherMail B.V.

This program is free software: you can redistribute it and/or modify it under
the terms of the GNU Affero General Public License version 3, 19 November 2007
as published by the Free Software Foundation.

This program is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
PARTICULAR PURPOSE. See the GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License along
with this program. If not, see <http://www.gnu.org/licenses/>.

Additional permission under GNU AGPL version 3 section 7:

If you modify this Program, or any covered work, by linking or combining it
with aspectjrt.jar, aspectjweaver.jar, tyrex-1.0.3.jar, freemarker.jar,
dom4j.jar, mx4j-jmx.jar, mx4j-tools.jar, spice-classman-1.0.jar,
spice-loggerstore-0.5.jar, spice-salt-0.8.jar, spice-xmlpolicy-1.0.jar,
wsdl4j-1.6.1.jar (or modified versions of these libraries), containing parts
covered by the terms of Eclipse Public License, tyrex license, freemarker
license, dom4j license, mx4j license, Spice Software License, Common
Development and Distribution License (CDDL), Common Public License (CPL) the
licensors of this Program grant you additional permission to convey the
resulting work.
