<#--
  This base template will be imported by templates which will be converted into MIME emails.

  The main reason for placing code in this file is that admins can modify the templates while we are flexible
  to add or change functionality to this file (the admin cannot change this file, at least not from the GUI),

  The top part of this template includes variables which are used by most email templates. Template specific
  functionality is added by using Freemarker macro's.

  Notes:
    - All content of a macro is added to the template which calls the macro. Newlines should therefore not be added
      because this will result in invalid MIME.
    - Variables which are cerated with assign can be used by a template and should therefore not be renamed or
      removed because a template might have been overridden by an admin
 -->

<#assign originatorProperties = (application.loadUserProperties(originator))!>
<#assign logo = (chunkString(originatorProperties.getProperty("email.resources.logo.company", false)))!>
<#assign footer = (originatorProperties.footer)!"Email encryption with support for S/MIME, OpenPGP, PDF Messenger and Webmail Messenger">
<#assign footerLink = (originatorProperties.footerLink)!"https://ciphermail.com">
<#assign footerLinkTitle = (originatorProperties.footerLinkTitle)!"www.ciphermail.com">

<#macro otp invite>
    <#assign qrCodeEnabled = (originatorProperties.pdfQrCodeEnabled)!false>
    <#local keyID = (passwordContainer.clientSecretID)!>
    <#-- passwordContainer variable is set by the PDFEncrypt Mailet -->
    <#assign passwordID = (passwordContainer.passwordID)!"not set">
    <#assign passwordLength = (passwordContainer.passwordLength)!0>
    <#-- create a QR code with a json value for the parameters -->
    <#assign qrCode = createQR("{\"app\":\"otp\",\"keyID\":\"" + keyID?json_string + "\",\"pwID\":\"" + passwordID?json_string + "\", \"pwLen\":" + passwordLength?json_string + "}", 200, 200, "png")>
    <#local baseURL = (originatorProperties.OTPURL)!>
    <#if baseURL?has_content>
        <#local recipient = recipients[0]>
        <#assign portal_url=baseURL+'?id='+passwordID+'&pwl='+passwordLength+'&email='+recipient?url('UTF-8')>
        <#if (invite!false)>
            <#-- portal invitation parameters are stored in the Mail attributes -->
            <#local portalInvitations = mail.getAttribute("djigzo.portalInvitations")>
            <#local portalInvitation = portalInvitations[recipient]>
            <#assign portal_url=portal_url+'&ts='+portalInvitation.timestamp?c+'&action=signup&mac='+portalInvitation.mac>
        </#if>
    </#if>
</#macro>

<#macro sms>
    <#assign passwordID = (passwordContainer.passwordID)!"not set">
</#macro>