#!/bin/bash -e

#
### BEGIN DJIGZO SCRIPT INFO
# Name: fetchmail
# Description: Script that configures Fetchmail by importing an XML file from standard input
### END DJIGZO SCRIPT INFO
#
# Author: Martijn Brinkers
#
# Copyright 2008-2021, CipherMail.
#
# Script that configures Fetchmail by importing an XML file from standard in containing the new Fetchmail 
# configuration. The new configuration is 'merged' with the fetchmail config file /etc/fetchmailrc by 
# replacing everything between ### START-AUTO-CONFIG ### and ### END-AUTO-CONFIG ### 

# set a sane/secure path
PATH='/bin:/usr/bin:/sbin:/usr/sbin'
# it's almost certainly already marked for export, but make sure
export PATH

# remove all aliases (start with \ to prevent unalias from being aliased)
\unalias -a

# clean command hash
hash -r

# set a sane/secure IFS (note this is bash & ksh93 syntax only--not portable!)
IFS=$' \t\n'

# Include defaults if available
if [ -f /etc/default/cm-fetchmail ] ; then
    . /etc/default/cm-fetchmail
fi

fetchmail_config="/etc/fetchmailrc"
fetchmail_config_new=$(mktemp) || exit 1

if [[ -z $jars ]]; then
    # find all jars which are own by root and are only writable by root
    jars="$(find lib/ \( -perm 644 -o -perm 664 \) -type f -name \*.jar -user root -group root | tr '\n' ':')"
fi

# write the new fetchmail config to temp file
java -cp "$jars" mitm.common.fetchmail.FetchmailConfigWriter "$fetchmail_config" > $fetchmail_config_new

# and overwrite the existing fetcmail config with the new file 
mv "$fetchmail_config_new" /etc/fetchmailrc

chown fetchmail:root /etc/fetchmailrc

# enable and restart fetchmail
systemctl enable fetchmail
systemctl restart fetchmail

exit 0
