#!/bin/bash -e

#
### BEGIN DJIGZO SCRIPT INFO
# Name: postfix-tls
# Description: Script that sets the tls server cert for postfx
### END DJIGZO SCRIPT INFO
#
# Author: Martijn Brinkers
#
# Copyright 2013-2021, CipherMail.
#

# set a sane/secure path
PATH='/bin:/usr/bin:/sbin:/usr/sbin'
# it's almost certainly already marked for export, but make sure
export PATH

# remove all aliases (start with \ to prevent unalias from being aliased)
\unalias -a

# clean command hash
hash -r

# set a sane/secure IFS (note this is bash & ksh93 syntax only--not portable!)
IFS=$' \t\n'

TLS_PEM="/etc/postfix/tls.pem"

usage()
{
    echo "usage: postfix-tls -s < STDIN" >&2

    exit 1
}

cleanup()
{
    if [ -n "$temp_in" ]; then
        rm "$temp_in"
    fi
}

restart_postfix()
{
    systemctl restart postfix
}

set_tls()
{
    # create temp pem files so we can filter the input
    temp_in=$(mktemp)

    cat > "$temp_in"

    # check whether the pem contains a certificate
    openssl x509 -in "$temp_in" > /dev/null
    # check whether the pem contains an RSA key
    openssl rsa -in "$temp_in" > /dev/null

    cp "$temp_in" "$TLS_PEM"

    chmod 600 "$TLS_PEM"

    restart_postfix
}

# create a trap that will cleanup all temporary resources
trap cleanup ABRT EXIT HUP INT QUIT

if [ $# -lt 1 ]; then
    usage
fi

while getopts 's' OPTION
do
    case $OPTION in
    s)  action="set"
        ;;
    ?)  usage
        ;;
    esac
done

shift $(($OPTIND - 1))

case $action in
    set) set_tls
    ;;
esac

exit 0
