#!/bin/bash -e

#
### BEGIN DJIGZO SCRIPT INFO
# Name: postfix
# Description: Script for managing postfix
### END DJIGZO SCRIPT INFO
#
# Author: Martijn Brinkers
#
# Copyright 2008-2021, CipherMail.
#

# set a sane/secure path
PATH='/bin:/usr/bin:/sbin:/usr/sbin'
# it's almost certainly already marked for export, but make sure
export PATH

# remove all aliases (start with \ to prevent unalias from being aliased)
\unalias -a

# clean command hash
hash -r

# set a sane/secure IFS (note this is bash & ksh93 syntax only--not portable!)
IFS=$' \t\n'

usage()
{
    echo "usage: postfix (postcat args | postfix args | postqueu args | postsuper args | start | stop | restart | running)" >&2

    exit 1
}

if [ $# -lt 1 ]; then
    usage
fi

stop_postfix()
{
    systemctl stop postfix
}

start_postfix()
{
    systemctl start postfix
}

restart_postfix()
{
    systemctl restart postfix
}

is_running()
{
    running='true'

    systemctl status postfix > /dev/null 2>&1 || running='false'

    echo "$running"
}

command=$1

shift

case "$command" in
    postcat)
        postcat $@ 
    ;;
    postfix)
        postfix $@ 
    ;;
    postqueue)
        postqueue $@ 
    ;;
    postsuper)
        postsuper $@ 
    ;;
    start)
        start_postfix
    ;;
    stop)
        stop_postfix
    ;;
    restart)
        restart_postfix
    ;;
    running)
        is_running
    ;;
    *)
        usage
esac

exit 0
