#!/bin/bash -e

#
### BEGIN DJIGZO SCRIPT INFO
# Name: ssl
# Description: Script for installing ssl certificate and key
### END DJIGZO SCRIPT INFO
#
# Author: Martijn Brinkers
#
# Copyright 2015-2018, CipherMail.
#

# set a sane/secure path
PATH='/bin:/usr/bin:/sbin:/usr/sbin'
# it's almost certainly already marked for export, but make sure
export PATH

# remove all aliases (start with \ to prevent unalias from being aliased)
\unalias -a

# clean command hash
hash -r

# set a sane/secure IFS (note this is bash & ksh93 syntax only--not portable!)
IFS=$' \t\n'

WEB_GUI_SSL_FILE="/usr/share/djigzo-web/ssl/sslCertificate.p12"
DEFAULT_PASSWORD="djigzo"

# Include defaults if available
if [ -f /etc/default/cm-ssl ] ; then
    . /etc/default/cm-ssl
fi

password="$DEFAULT_PASSWORD"

cleanup()
{
    if [ -f "$tmpfile" ]; then
        rm $tmpfile
    fi
}

usage()
{
    echo "usage: $0 -i" >&2

    exit 1
}

if [ $# -lt 1 ]; then
    usage
fi

install_pfx()
{
    tmpfile=$(mktemp) || exit 1

    cat > "$tmpfile"

    cp "$tmpfile" "$WEB_GUI_SSL_FILE"

    chmod 640 "$WEB_GUI_SSL_FILE"
}

# create a trap that will cleanup all temporary resources
trap cleanup ABRT EXIT HUP INT QUIT

while getopts 'i' OPTION
do
    case $OPTION in
    i)  action="install"
        ;;
    ?)  usage
        ;;
    esac
done

shift $(($OPTIND - 1))

case $action in
    install) install_pfx
        ;;
esac

exit 0

