#!/bin/bash -e

set -o pipefail

# Starts the CipherMail back-end
#
# Copyright CipherMail 2021

# set a sane/secure path
PATH='/bin:/usr/bin:/sbin:/usr/sbin'
# it's almost certainly already marked for export, but make sure
export PATH

# remove all aliases (start with \ to prevent unalias from being aliased)
\unalias -a

# clean command hash
hash -r

# set a sane/secure IFS (note this is bash & ksh93 syntax only--not portable!)
IFS=$' \t\n'

# the console should be set to UTF-8 in order to support multibyte characters
export LANG=en_US.UTF-8
export LC_CTYPE="en_US.UTF-8"

wrapper_opts=""

# include defaults if available
if [ -f /etc/default/cm-gateway-backend-start ] ; then
    . /etc/default/cm-gateway-backend-start
fi

if [[ -z $ciphermail_home ]]; then
    ciphermail_home="/usr/share/djigzo"
fi

if [[ -z $backend_bin ]]; then
    backend_bin="$ciphermail_home/wrapper/wrapper-djigzo"
fi

if [[ -z $wrapper_config ]]; then
    # wrapper config is relative to the wrapper dir
    wrapper_config=../../djigzo.wrapper.conf
fi

if [[ -z $max_jvm_mem ]]; then
    # max heap mem for jvm 16GB
    max_jvm_mem="16384"
fi

if [[ -z $perc_jvm_mem ]]; then
    # percentage of total mem available for jvm
    perc_jvm_mem="0.6"
fi

function debug()
{
    if [[ $debug = true ]]; then
        echo "$1" >&2
    fi
}

# 'dynamic' memory allocation
# set max memory based on the total available memory - while reserve some for other processes

# get total_memory in MB
total_memory_mb=$(cat /proc/meminfo | grep "MemTotal:" | awk '{print int($2 / 1024)}')

jvm_max_memory_mb=$(awk 'BEGIN{allocate='"$total_memory_mb"'*'"$perc_jvm_mem"'; if (allocate > '"$max_jvm_mem"') { allocate='"$max_jvm_mem"' } {printf "%d",allocate}}')

debug "jvm_max_memory_mb: $jvm_max_memory_mb"

wrapper_opts="$wrapper_opts wrapper.java.maxmemory=$jvm_max_memory_mb"

$backend_bin $wrapper_config "$wrapper_opts"

exit 0

